/*
** Copyright (C) 1998,1999,2000,2001 Martin Roesch <roesch@clark.net>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* $Id: avl_tree.c,v 1.10 2001/06/16 20:41:16 roesch Exp $ */
#include <stdlib.h>
#include <stdio.h>
#include "avl_tree.h"

int avlnodes;

void inserttree(int newdata, struct Node ** start, int maxdepth)
{
    struct Node *tmp;

    if (*start == NULL)   
    {
        tmp = (struct Node *) calloc(sizeof(struct Node), sizeof(char));

        if(!tmp)
        {  
            printf("insertnode failed, Not enough memory!\n");
            exit(1);    
        }

        tmp->left = NULL;
        tmp->right = NULL;
        tmp->data = newdata;
        tmp->balance = 0;
        tmp->nodes = 1;
        *start = tmp;
        avlnodes++;
    }
    else
        insertnode(newdata, start, 1, maxdepth);
}


void insertnode(int newdata, struct Node ** start, int depth, int maxdepth)
{
    struct Node *tmp;

    if(newdata == (*start)->data)     /* don't add duplicate  */
    {
        LogMessage("AVL keyspace collision, please let Marty know\n");
        return;
    }

    if(depth >= maxdepth)
    { 
        LogMessage("max nodes reach, data is not inserted\n");
        return;
    }

    if(newdata < (*start)->data && (*start)->left != NULL) 
    {
        insertnode(newdata, &(*start)->left, depth+1, maxdepth);
    }
    else if(newdata > (*start)->data && (*start)->right != NULL)
    {
        insertnode(newdata, &(*start)->right, depth+1, maxdepth);
    }
    else if(newdata < (*start)->data && (*start)->left == NULL)
    {
        tmp = (struct Node *) calloc(sizeof(struct Node), sizeof(char));

        if(!tmp)
        {  
            printf("insertnode failed, Not enough memory!\n");
            exit(1);    
        }

        tmp->left = NULL;
        tmp->right = NULL;
        tmp->data = newdata;
        tmp->balance = 0;
        tmp->nodes = 1;
        (*start)->left = tmp;
        avlnodes++;
    }
    else 
    {
        tmp = (struct Node *) malloc(sizeof(struct Node));

        if(!tmp)
        {  
            printf("insertnode failed, Not enough memory!\n");
            exit(1);    
        }

        tmp->left = NULL;
        tmp->right = NULL;
        tmp->data = newdata;
        tmp->balance = 0;
        tmp->nodes = 1;
        (*start)->right = tmp;
        avlnodes++;
    }

    checkbalance(*start);

    if((*start)->balance >= 2)
        balanceright(&(*start));

    if((*start)->balance <= -2)
        balanceleft(&(*start));
}



void deletenode(int newdata, struct Node ** start, struct Node **parent)
{
    struct Node *tmp, *replparent;

    if ((*start) == NULL)
        return;

    if ((*start)->data == newdata)              /*start is node to be deleted */
    {
        if ((*start)->left == NULL && (*start)->right == NULL)
        {
            if (newdata < (*parent)->data)
                (*parent)->left = NULL;
            else
                (*parent)->right = NULL;
#ifdef DEBUG
            printf("freeing AVL node\n");
#endif
            if((*start)->nsptr != NULL)
                free((*start)->nsptr);
            free(*start);
            *start = NULL;
            avlnodes--;
        }
        else if ((*start)->left == NULL)                 /*right branch exits */
        {
            if (newdata < (*parent)->data)
                (*parent)->left = (*start)->right;
            else
                (*parent)->right = (*start)->right;
#ifdef DEBUG
            printf("freeing AVL node\n");
#endif
            if((*start)->nsptr != NULL)
                free((*start)->nsptr);
            free(*start);
            *start = NULL;
            avlnodes--;
        }
        else if ((*start)->right == NULL)                 /*left branch exits */
        {
            if (newdata < (*parent)->data)
                (*parent)->left = (*start)->left;
            else
                (*parent)->right = (*start)->left;
#ifdef DEBUG
            printf("freeing AVL node\n");
#endif
            if((*start)->nsptr != NULL)
                free((*start)->nsptr);
            free(*start);
            *start = NULL;
            avlnodes--;
        }
        else
        {                                           /* has 2 children  */ 
            /* replace the node with the smallest value in the right tree */

            replparent = (*start);   
            tmp = (*start)->right;

            while (tmp->left != NULL)
            {
                replparent = tmp;
                tmp = tmp->left;
            } 

            /* we have smallest value */

            (*start)->data = tmp->data;
            if (tmp->data < replparent->data)
                replparent->left = tmp->right;
            else
                replparent->right = tmp->right;
#ifdef DEBUG
            printf("freeing AVL node\n");
#endif
            if(tmp->nsptr != NULL)
                free(tmp->nsptr);
            free(tmp);
            avlnodes--;
        }  
    }

    else if(newdata > (*start)->data && (*start)->right != NULL)
        deletenode(newdata, &(*start)->right, &(*start));

    else if(newdata < (*start)->data && (*start)->left != NULL)
        deletenode(newdata, &(*start)->left, &(*start));

    if (*start != NULL)
    {
        checkbalance(*start);

        if((*start)->balance >= 2)
            balanceright(&(*start));

        if(*start != NULL)
        {
            if((*start)->balance <= -2)
                balanceleft(&(*start));
        }
    }
}

void checkbalance(struct Node *start)
{
    int lnodes, rnodes;
    if (start == NULL)
        return;

    if (start->left == NULL)
        lnodes = 0;
    else
        lnodes = start->left->nodes;

    if (start->right == NULL)
        rnodes = 0;
    else
        rnodes = start->right->nodes;


    if(lnodes > rnodes)
        start->nodes = lnodes + 1;
    else
        start->nodes = rnodes + 1;

    start->balance = rnodes - lnodes;

}
   
void balanceright(struct Node ** start)
{

    if ((*start)->right->balance >= 1)
        rotateleft(&(*start));
    else
    {
        rotateright(&(*start)->right);
        rotateleft(&(*start));
    }

}

void balanceleft(struct Node ** start)
{
    if ((*start)->left->balance <= -1)
        rotateright(&(*start));
    else
    {
        rotateleft(&(*start)->left);
        rotateright(&(*start));
    }

}


void rotateright(struct Node **start)
{
    struct Node *tmp;
    tmp = *start;
    *start = (*start)->left;
    if(*start == NULL)
        return;
    tmp->left = (*start)->right;
    (*start)->right = tmp;
    checkbalance((*start)->left);
    checkbalance((*start)->right);
    checkbalance(*start);
}    

void rotateleft(struct Node ** start)
{
    struct Node *tmp;
    tmp = *start;
    *start = (*start)->right;
    if(*start == NULL)
        return;
    tmp->right = (*start)->left;
    (*start)->left = tmp;
    checkbalance((*start)->left);
    checkbalance((*start)->right);
    checkbalance(*start);

}

struct Node * findnode(struct Node *start, int newdata)
{
    if (start == NULL)
        return NULL;
    if (start->data == newdata)
        return start;
    if (newdata < start->data)
        return findnode(start->left, newdata);
    else
        return findnode(start->right, newdata);
}

void printavltree( struct Node *start)
{
    if (start == NULL) return;
    printavltree(start->left);
    printf("p: %d lc: %d rc: %d\n", start->data, (start->left) == NULL ? 0: start->left->data, (start->right) == NULL ? 0: start->right->data);
    printavltree(start->right);

}

void walkavltree(void (*callback)(struct Node*), struct Node *start)
{
    if (start == NULL) return;
    walkavltree(callback, start->left);
        (*callback)(start);
    walkavltree(callback, start->right);
}

#ifdef DEBUG
void countnodes( struct Node *start)
{
    int nodecount;

    if (start == NULL) return;
    countnodes(start->left);
    nodecount++;
    countnodes(start->right);
}
#endif
