/*
** Copyright (C) 1998,1999,2000,2001 Martin Roesch <roesch@clark.net>
** Copyright (C) 1999,2000,2001 Christopher E. Cramer <cec@ee.duke.edu>
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* Snort TCP Stream Reconstruction Plugin v3
   by Martin Roesch <roesch@clark.net> and 
   Christopher E. Cramer <cec@ee.duke.edu>
*/

/* $Id: spp_tcp_stream3.h,v 1.4 2001/05/12 20:25:10 roesch Exp $ */

#include <time.h> /* time_t */
#include "snort.h"
#include "avl_tree.h"

#ifndef __SPP_TCP_STREAM3_H__
#define __SPP_TCP_STREAM3_H__

#define CLOSED       0
#define LISTEN       1
#define SYN_RCVD     2
#define SYN_SENT     3
#define ESTABLISHED  4
#define CLOSE_WAIT   5
#define LAST_ACK     6
#define FIN_WAIT_1   7
#define CLOSING      8
#define FIN_WAIT_2   9
#define TIME_WAIT   10

#define COLLECT_CLIENT 0x01
#define COLLECT_SERVER 0x02

typedef struct _TcpStream3PacketData
{
    u_int8_t   *data;
    u_int16_t size;

    u_int32_t seq;

    u_int32_t id;  /* debug var */

    struct _TcpStream3PacketData *next;
    struct _TcpStream3PacketData *prev;
} TcpStream3PacketData;

typedef struct _TcpStream3Session
{
    u_int32_t sip;      /* server IP */
    u_int16_t sp;       /* server port */
    u_int32_t server_status;
    u_int32_t s_first_seq;
    u_int32_t s_last_ack;
    u_int32_t s_win;

    u_int16_t s_stream_size;
    u_int16_t s_count;  /* number of PacketData nodes being held */
    TcpStream3PacketData *s_data;
    TcpStream3PacketData *s_data_end;

    u_int32_t cip;      /* client ip */
    u_int16_t cp;       /* client port */
    u_int32_t client_status; 
    u_int32_t c_first_seq;  
    u_int32_t c_last_ack;
    u_int32_t c_win;

    u_int16_t c_stream_size;
    u_int16_t c_count;  /* number of PacketData nodes being held */
    TcpStream3PacketData *c_data;
    TcpStream3PacketData *c_data_end;

    u_int32_t timestamp;  /* last access time */

    struct _TcpStream3Session *next;

        time_t itime;
        int nsession;
} TcpStream3Session;

typedef struct _TcpStream3Data
{
    u_int16_t ports[32];   /* 32 is kind of arbitrary */
    u_char port_collect[32];
    u_int32_t num_entries;
    u_char trunc;
    u_int32_t timestamp;
    u_int32_t prunetime;
    u_int32_t treedepth;
} TcpStream3Data;


struct Node{
  int nodes, balance;
  int data;
  TcpStream3Session *nsptr;
  struct Node *left;
  struct Node *right;
};

struct Node *rootnode;


/* list of function prototypes for this preprocessor */
void SetupTcpStream3();
void TcpStream3Init(u_char *);
void ParseTcpStream3Args(char *);
void TcpStream3Packet(Packet *);
u_long TcpStream3Time(Packet *);
TcpStream3Session * TcpStream3CheckSession(Packet *, int, int, int * );
void TcpStream3PruneTree(struct Node *, Packet *);
void TcpStream3PruneSession(int, TcpStream3Session *);
void TcpStream3Commit(TcpStream3Session *, int );
void TcpStream3Packetize(TcpStream3Session *, Packet *, u_long , int);
void TcpStream3Exit(int signal, void *arg);
void TcpStream3Restart(int signal, void *arg);
u_long TcpStream3FillBuffer(TcpStream3Session *, u_char *, u_long, int);
void TcpStream3Packetize(TcpStream3Session *, Packet *, u_long, int);
void TcpStream3StoreData( TcpStream3Session *, Packet *, int);
u_long TcpStream3CheckAck(TcpStream3Session *, u_long, int);
u_long TcpStream3DataSize(TcpStream3Session *, int);
void TcpStream3PrintDataList(TcpStream3Session *, int);
int IsWatchingPort(int, int, int, int);


#endif  /* __SPP_TCP_STREAM3_H__ */
